@@ -11,6 +11,7 @@ from rich.prompt import Confirm
 from yaspin import yaspin
 
 from grid.cli import rich_click
+from grid.cli.exceptions import GridError
 from grid.cli.observables import BaseObservable
 from grid.cli.rich_click import deprecate_option
 from grid.sdk import env
@@ -76,8 +77,8 @@ def datastore(ctx, cluster_id: str, is_global: bool, show_incomplete: bool) -> N
                 incomplete_id = find_incomplete_datastore_upload(grid_dir=Path(env.GRID_DIR))
                 if incomplete_id is not None:
                     ds = load_datastore_work_state(grid_dir=Path(env.GRID_DIR), datastore_id=incomplete_id)
-                    created_at = f"{datetime.fromtimestamp(ds.creation_timestamp):%Y-%m-%d %H:%M}"
-                    table_rows.append([ds.datastore_name, ds.cluster_id, created_at, ds.source])
+                    created = f"{datetime.fromtimestamp(ds.creation_timestamp):%Y-%m-%d %H:%M}"
+                    table_rows.append([ds.datastore_name, ds.cluster_id, created, ds.source])
             except Exception as e:
                 spinner.fail("✘")
                 raise click.ClickException(e)
@@ -85,18 +86,32 @@ def datastore(ctx, cluster_id: str, is_global: bool, show_incomplete: bool) -> N
             spinner = yaspin(text=f"Loading Datastores in {env.CONTEXT}...", color="yellow")
             spinner.start()
 
-            table_cols = ["Name", "Cluster ID", "Version", "Size", "Created At", "Status"]
             try:
                 datastores: List[Datastore] = list_datastores(cluster_id=cluster_id, is_global=is_global)
             except Exception as e:
                 spinner.fail("✘")
-                raise click.ClickException(e)
+                raise click.ClickException(str(e))
 
-            for ds in sorted(datastores, key=lambda k: (k.name, k.version)):
-                created_at = f'{ds.created_at:%Y-%m-%d %H:%M}'
-                size = ds.size
-                status = ds.snapshot_status
-                table_rows.append([ds.name, ds.cluster_id, str(ds.version), size, created_at, status])
+            if is_global is True:
+                table_cols = [
+                    "Name", "Cluster ID", "Version", "Size", "Created At", "Created By", "Team Name", "Status"
+                ]
+                for ds in sorted(datastores, key=lambda k: (k.name, k.version)):
+                    created = f'{ds.created_at:%Y-%m-%d %H:%M}'
+                    size = ds.size
+                    owner = ds.user.username
+                    team = ds.user.team_name
+                    status = ds.snapshot_status
+                    status = status.replace("DATASTORE_OPTIMIZATION_STATUS_", "").lower()
+                    table_rows.append([ds.name, ds.cluster_id, str(ds.version), size, created, owner, team, status])
+            else:
+                table_cols = ["Name", "Cluster ID", "Version", "Size", "Created At", "Status"]
+                for ds in sorted(datastores, key=lambda k: (k.name, k.version)):
+                    created = f'{ds.created_at:%Y-%m-%d %H:%M}'
+                    size = ds.size
+                    status = ds.snapshot_status
+                    status = status.replace("DATASTORE_OPTIMIZATION_STATUS_", "").lower()
+                    table_rows.append([ds.name, ds.cluster_id, str(ds.version), size, created, status])
 
         table = BaseObservable.create_table(columns=table_cols)
         for row in table_rows:
@@ -150,75 +165,6 @@ def _resume():
             raise click.ClickException(f"no incomplete datastore upload sessions exist")
 
 
-@datastore.command()
-@rich_click.argument(
-    'source',
-    type=str,
-    default=None,
-    required=False,
-    help=(
-        "Source to create datastore from. This could either be a local "
-        "directory (e.g: /opt/local_folder) a remote http URL pointing "
-        "to a TAR or ZIP file (e.g. http://some_domain/data.tar.gz), or "
-        "an s3 bucket to copy data from (e.g. s3://ryft-public-sample-data/esRedditJson/)"
-    )
-)
-@click.option(
-    '--source',
-    'source_',
-    type=str,
-    show_default=False,
-    default=None,
-    required=False,
-    hidden=True,
-    callback=partial(deprecate_option, "the argument form (grid datastore create SOURCE) of this command"),
-)
-@click.option('--name', type=str, required=False, help='Name of the datastore')
-@click.option(
-    '--cluster',
-    type=str,
-    default=env.CONTEXT,
-    show_default=True,
-    required=False,
-    help='cluster id to create the datastore on. (Bring Your Own Cloud Customers Only).'
-)
-@click.pass_context
-def create(ctx, source: str, source_: str, cluster: str, name: Optional[str] = None) -> None:
-    """Creates a datastore from SOURCE.
-
-    The upload session is referenced by the name. this name
-    must be used to resume the upload if it is interupted.
-    """
-    if source_ is not None:
-        source = source_
-    try:
-        Console()  # THIS IS IMPORTANT! (otherwise console text overwrites eachother)
-        incomplete_id = find_incomplete_datastore_upload(Path(env.GRID_DIR))
-        if incomplete_id is not None:
-            should_resume = Confirm.ask(
-                "[b]A previous datastore upload was interrupted before it was able to complete.[/b]\n\n"
-                "Do you wish to resume this upload [bold green](yes/y)[reset]? If not [bold red](no/n)[reset], \n"
-                "then the [i]progress on the interrupted upload will be deleted[/i], \n"
-                "and you will have to upload the dataset in full.\n",
-                default=True,
-            )
-            if should_resume is True:
-                _resume()
-            else:
-                remove_datastore_work_state(grid_dir=Path(env.GRID_DIR), datastore_id=incomplete_id)
-                dstore = Datastore(name=name, source=source, cluster_id=cluster)
-                click.echo(f"creating datastore from {source}")
-                dstore.upload()
-        else:
-            with Console().status("[bold yellow]Indexing datastore uploads....", spinner_style="yellow") as status:
-                dstore = Datastore(name=name, source=source, cluster_id=cluster)
-                click.echo(f"creating datastoree from {source}")
-                status.stop()
-                dstore.upload()
-    except Exception as e:
-        raise click.ClickException(e)
-
-
 @datastore.command()
 @click.pass_context
 def clearcache(ctx) -> None:
@@ -227,7 +173,9 @@ def clearcache(ctx) -> None:
     This removes all the cached files from the local machine, meaning that resuming an incomplete
     upload is not possible after running this command.
     """
-    for f in Path(env.GRID_DIR).joinpath("datastores").iterdir():
+    grid_dstore_path = Path(env.GRID_DIR).joinpath("datastores")
+    grid_dstore_path.mkdir(parents=True, exist_ok=True)
+    for f in grid_dstore_path.iterdir():
         if f.is_file():
             os.remove(str(f.absolute()))
         if f.is_dir():
@@ -259,3 +207,112 @@ def delete(ctx, name: str, version: int, cluster: str) -> None:
     except Exception as e:
         raise click.ClickException(str(e))
     click.echo("Done!")
+
+
+@datastore.command()
+@rich_click.argument(
+    'source',
+    type=str,
+    default=None,
+    required=False,
+    help=(
+        "Source to create datastore from. This could either be a local "
+        "directory (e.g: /opt/local_folder) a remote http URL pointing "
+        "to a TAR or ZIP file (e.g. http://some_domain/data.tar.gz), or "
+        "an s3 bucket to copy data from (e.g. s3://ryft-public-sample-data/esRedditJson/)"
+    )
+)
+@click.option(
+    '--source',
+    'source_',
+    type=str,
+    show_default=False,
+    default=None,
+    required=False,
+    hidden=True,
+    callback=partial(deprecate_option, "the argument form (grid datastore create SOURCE) of this command"),
+)
+@click.option(
+    "--no-copy",
+    "_no_copy_data_source",
+    is_flag=True,
+    default=False,
+    help=(
+        "(beta) Use this flag when you intend to incrementally add data to the source bucket. "
+        "Using this flag can also speed up datastore creation when working with large buckets. "
+        "When using this flag, you cannot remove files from your bucket. If you'd like to add "
+        "files, please create a new version of the datastore after you've added files to your "
+        "bucket. Please note that Grid does not currently support private S3 buckets."
+    ),
+)
+@click.option('--name', type=str, required=False, help='Name of the datastore')
+@click.option(
+    '--cluster',
+    type=str,
+    default=env.CONTEXT,
+    show_default=True,
+    required=False,
+    help='cluster id to create the datastore on. (Bring Your Own Cloud Customers Only).'
+)
+@click.pass_context
+def create(
+    ctx, source: str, source_: str, cluster: str, _no_copy_data_source: bool, name: Optional[str] = None
+) -> None:
+    """Creates a datastore from SOURCE.
+
+    If you want to check the status of your datastore creation, please use the
+    `grid datastore` command.
+
+    Here are some examples of this command in use:
+
+    To create a datastore from a directory (or file) on the local machine
+    (optionally specifying a name):
+
+        grid datastore create ./my_cool_file.txt
+
+        grid datastore create ./some-directory/
+
+        grid datastore create ./some-directory/ --name my-awesome-datastore
+
+    To create a datastore from an S3 bucket (private buckets are not currently
+    supported):
+
+        grid datastore create s3://ryft-public-sample-data/esRedditJson/
+
+    If you'd like to create a datastore from an S3 bucket that will be
+    incrementally updated, or for very large datasets:
+
+        grid datastore create s3://ryft-public-sample-data/ --no-copy
+
+    To create a datastore from a zip or tar.gz file hosted at a URL:
+
+        grid datastore create https://cs.nyu.edu/~roweis/data/EachMovieData.tar.gz
+    """
+    if source_ is not None:
+        source = source_
+    try:
+        Console()  # THIS IS IMPORTANT! (otherwise console text overwrites eachother)
+        incomplete_id = find_incomplete_datastore_upload(Path(env.GRID_DIR))
+        if incomplete_id is not None:
+            should_resume = Confirm.ask(
+                "[b]A previous datastore upload was interrupted before it was able to complete.[/b]\n\n"
+                "Do you wish to resume this upload [bold green](yes/y)[reset]? If not [bold red](no/n)[reset], \n"
+                "then the [i]progress on the interrupted upload will be deleted[/i], \n"
+                "and you will have to upload the dataset in full.\n",
+                default=True,
+            )
+            if should_resume is True:
+                _resume()
+            else:
+                remove_datastore_work_state(grid_dir=Path(env.GRID_DIR), datastore_id=incomplete_id)
+                dstore = Datastore(name=name, source=source, cluster_id=cluster, s3_no_copy=_no_copy_data_source)
+                click.echo(f"\ncreating datastore from {source}")
+                dstore.upload()
+        else:
+            with Console().status("[bold yellow]Indexing datastore uploads....", spinner_style="yellow") as status:
+                dstore = Datastore(name=name, source=source, cluster_id=cluster, s3_no_copy=_no_copy_data_source)
+                click.echo(f"\ncreating datastore from {source}")
+                status.stop()
+                dstore.upload()
+    except Exception as e:
+        raise click.ClickException(e)
