@@ -142,9 +142,16 @@ def _run_command_builder(ctx, _param, value):
 @click.option(
     '--strategy',
     'strategy',
-    type=click.Choice(['grid_search', 'random_search'], case_sensitive=False),
+    type=click.Choice(['grid_search', 'random_search', 'none'], case_sensitive=False),
     required=False,
-    help='Hyper-parameter search strategy'
+    default="grid_search",
+    help="""Hyper-parameter search strategy\b
+
+Available options:
+ - grid_search (creates invocations for each combinations or evaluated arguments)
+ - random_search (creates random_search conbination of parameters - you can add num_trials for defining exact number of them)
+ - none (script arguments will not be evaluated at all and passed directly to the script)
+"""
 )
 @click.option(
     '--num_trials',
@@ -164,9 +171,8 @@ def _run_command_builder(ctx, _param, value):
     '--instance_type',
     'instance_type',
     type=str,
-    default='t2.medium',
+    default='m5a.large',
     help='Instance type to start training session in',
-    autocompletion=_get_instance_types,
     callback=_resolve_instance_type_nickname
 )
 @click.option('--gpus', 'gpus', type=int, required=False, default=0, help='Number of GPUs to allocate per experiment')
@@ -194,23 +200,8 @@ def _run_command_builder(ctx, _param, value):
     'framework',
     required=False,
     default='lightning',
-    help="""Framework to use in training.\b
-
-Select from available options:
- - lightning
- - torch
- - tensorflow
- - julia (will select the latest available version).
- - julia:1.6.1
- - julia:1.6.2
- - julia:1.6.3
- - julia:1.6.4
- - julia:1.6.5
- - julia:1.7.0
- - julia:1.7.1
- - torchelastic
-
-""",
+    help=
+    "Framework to use in training. Select from available options: `lightning`, `torch`, `tensorflow`, `julia (will select the latest available version)`, `julia:1.6.1`, `julia:1.6.2`, `julia:1.6.3`, `julia:1.6.4`, `julia:1.6.5`, `julia:1.7.0`, `julia:1.7.1`, `torchelastic`",  # noqa
 )
 @click.option(
     '--use_spot',
@@ -317,6 +308,7 @@ def run(
     # Setting this to the global variable so downstream tasks can pick it up easily
     if ignore_warnings:
         env.IGNORE_WARNINGS = ignore_warnings
+    validate_xpus(instance_type=instance_type, xpu=gpus or cpus)
 
     if env.DEBUG:
         click.echo(f"Run command: {run_command}")
@@ -398,3 +390,20 @@ def run(
         click.echo(message)
     else:
         click.echo(f"Run {name} failed to submit")
+
+
+def validate_xpus(instance_type: str, xpu: int) -> None:
+    if not instance_type:
+        return
+
+    aws_machine_name = _aws_node_to_nickname().get(instance_type, None)
+    if not aws_machine_name:
+        return
+
+    machine_xpus = aws_machine_name.split('_')[0]
+    assert machine_xpus.isdigit(), "aws machine names formatting is not correct, it should be <xpu-count>_name_<memory>"
+
+    if xpu and int(machine_xpus) < xpu:
+        raise click.exceptions.ClickException(
+            f"You have specified {xpu} GPUs (or CPUs) which is more than available on your selected instance type.\nPlease specify up to {int(machine_xpus)} GPUs and try again."
+        )
