@@ -51,10 +51,9 @@ def session(ctx, is_global: bool) -> None:
     '--instance_type',
     'instance_type',
     type=str,
-    default='t2.medium',
+    default='m5a.large',
     callback=_resolve_instance_type_nickname,
     help='Instance type to start session in.',
-    autocompletion=_get_instance_types
 )
 @click.option(
     '--use_spot',
@@ -255,7 +254,9 @@ def delete(session_name: str) -> None:
 
 @session.command()
 @rich_click.argument('session_name', type=str, nargs=1, help='Name of the session to change')
-@rich_click.argument('instance_type', type=str, nargs=1, help='Instance type to change to')
+@rich_click.argument(
+    'instance_type', type=str, nargs=1, help='Instance type to change to', callback=_resolve_instance_type_nickname
+)
 @click.option(
     '--spot',
     type=bool,
