@@ -1,11 +1,13 @@
 import textwrap
 import warnings
+from collections import defaultdict
 from datetime import datetime, time
 from pathlib import Path
-from typing import List, Optional, Union
+from typing import List, Optional, Union, Dict
 from urllib.parse import urlparse
 
 import click
+import rich
 
 from grid.openapi import V1GetDatastoreResponse, V1DatastoreOptimizationStatus, V1DatastoreSourceType
 from grid.openapi.rest import ApiException
@@ -91,6 +93,7 @@ class Datastore:
     _id: str
     _version: int
     _source: Optional[Union[str, Path]]
+    _s3_no_copy: bool
     _snapshot_status: str
     _created_at: datetime
     _user: User
@@ -105,6 +108,7 @@ class Datastore:
         self,
         name: Optional[str] = None,
         source: Optional[Union[str, Path]] = None,
+        s3_no_copy: bool = False,
         user: Optional[User] = None,
         version: int = 0,
         cluster_id: Optional[str] = None,
@@ -133,6 +137,8 @@ class Datastore:
             The version of the DataStore.
         source
             The location of the DataStore on disk or at a remote location.
+        s3_no_copy
+            If using an s3:// bucket as source use a remote reference type
         user
             The user that owns the DataStore.
         cluster_id
@@ -191,6 +197,7 @@ class Datastore:
         self._source = source
         self._user = user
         self._cluster_id = cluster_id
+        self._s3_no_copy = s3_no_copy
         self._id = None
         self._snapshot_status = None
         self._created_at = None
@@ -218,6 +225,7 @@ class Datastore:
         instance._cluster_id = datastore.spec.cluster_id
         instance._version = datastore.spec.version
         instance._source = datastore.spec.source
+        instance._s3_no_copy = instance._source == V1DatastoreSourceType.OBJECT_STORE_REFERENCE_ONLY
         instance._snapshot_status = str(datastore.status.phase)  # TODO - rename to status
         instance._created_at = datastore.created_at
         instance._size = f"{datastore.spec.size_mib} MiB"
@@ -235,6 +243,14 @@ class Datastore:
     def id(self) -> str:
         return self._id
 
+    @property
+    @affirm(is_not_shallow, is_not_deleted)
+    def s3_no_copy(self):
+        """When using an s3:// bucket source, this flag determines if we the data should
+        be copied into the grid managed bucket, or if we should only keep a reference to it.
+        """
+        return self._s3_no_copy
+
     # ------------------ Attributes Only Valid Before Upload ---------------
 
     @property
@@ -349,11 +365,12 @@ class Datastore:
             res = textwrap.dedent(
                 f"""\
                 {self.__class__.__name__}(
-                    {"name": <10} = \"{self.name}\",
-                    {"version": <10} = {self.version},
-                    {"source": <10} = \"{self.source}\",
-                    {"owner": <10} = {self.user},
-                    {"cluster_id": <10} = {self.cluster_id},
+                    {"name": <12} = \"{self.name}\",
+                    {"version": <12} = {self.version},
+                    {"source": <12} = \"{self.source}\",
+                    {"s3_no_copy": <12} = {self.s3_no_copy},
+                    {"owner": <12} = {self.user},
+                    {"cluster_id": <12} = {self.cluster_id},
                 )"""
             )
         return res
@@ -410,7 +427,13 @@ class Datastore:
         ``Datastore`` object again with the same parameters repeating the call to ``upload()``.
         """
         self._client = GridRestClient(create_swagger_client())
-        dstore = create_datastore(c=self._client, cluster_id=self._cluster_id, name=self._name, source=self._source)
+        dstore = create_datastore(
+            c=self._client,
+            cluster_id=self._cluster_id,
+            name=self._name,
+            source=self._source,
+            no_copy_data_source=self._s3_no_copy
+        )
         self._id = dstore.id
         self._version = dstore.spec.version
         self._created_at = dstore.created_at
@@ -454,36 +477,41 @@ def list_datastores(cluster_id: Optional[str] = None, is_global: bool = False) -
     cluster_id:
         if specified, returns a list of datastores for the specified cluster
     """
-    datastores = []
-    team_user_id_name_map = {}
-    client = GridRestClient(create_swagger_client())
-    cluster_id = cluster_id or env.CONTEXT
-
-    user_datastore_resps = get_datastore_list(client=client, cluster_id=cluster_id)
     user = user_from_logged_in_account()
-    team_user_id_name_map[user.user_id] = user.username
-    for user_datastore in user_datastore_resps:
-        dstore_resp = get_datastore_from_id(
-            c=client, datastore_id=user_datastore.id, cluster_id=user_datastore.spec.cluster_id
-        )
-        dstore = Datastore._setup_from_response(datastore=dstore_resp)
-        dstore.user.username = user.username
-        datastores.append(dstore)
+    client = GridRestClient(create_swagger_client())
 
-    # If ``include_teams`` is set, add datastores registered to the team.
-    team_user_ids = []
-    if is_global:
-        teams_data = get_user_teams()
-        for team_data in teams_data:
+    cluster_id = cluster_id or env.CONTEXT
+    datastores = []
+
+    if not is_global:
+        # single user only
+        user_datastore_resps = get_datastore_list(client=client, cluster_id=cluster_id)
+        for user_datastore in user_datastore_resps:
+            dstore_resp = get_datastore_from_id(
+                c=client, datastore_id=user_datastore.id, cluster_id=user_datastore.spec.cluster_id
+            )
+            dstore = Datastore._setup_from_response(datastore=dstore_resp)
+            dstore.user.username = user.username
+            datastores.append(dstore)
+    else:
+        # If ``include_teams`` is set, add datastores registered to the team.
+        team_user_id_name_map: Dict[str, Dict[str, str]] = {}
+        team_user_id_name_map[user.user_id] = {"username": user.username, "team_name": ""}
+        for team_data in get_user_teams():
             for member_data in team_data['members']:
-                team_user_ids.append(member_data['id'])
-                team_user_id_name_map[member_data['id']] = member_data['username']
+                team_user_id_name_map[member_data['id']] = {
+                    "username": member_data['username'],
+                    "team_name": team_data['name'],
+                }
 
-        team_dstore_list = get_datastore_list(client=client, cluster_id=cluster_id, user_ids=team_user_ids)
+        team_dstore_list = get_datastore_list(
+            client=client, cluster_id=cluster_id, user_ids=list(team_user_id_name_map.keys())
+        )
         for team_dstore in team_dstore_list:
             resp = get_datastore_from_id(c=client, cluster_id=team_dstore.spec.cluster_id, datastore_id=team_dstore.id)
             dstore = Datastore._setup_from_response(datastore=resp)
-            dstore.user.username = team_user_id_name_map[dstore.user.user_id]
+            dstore.user.username = team_user_id_name_map[dstore.user.user_id]['username']
+            dstore.user.team_name = team_user_id_name_map[dstore.user.user_id]['team_name']
             datastores.append(dstore)
 
     return datastores
