@@ -26,7 +26,12 @@ if TYPE_CHECKING:
 
 @throw_with_message
 def create_datastore(
-    c: GridRestClient, cluster_id: str, name: str, source: str, version: int = 0
+    c: GridRestClient,
+    cluster_id: str,
+    name: str,
+    source: str,
+    no_copy_data_source: bool,
+    version: int = 0,
 ) -> V1CreateDatastoreResponse:
     """Create a new datastore.
 
@@ -40,22 +45,36 @@ def create_datastore(
         Human readable name assigned to the datastore.
     source
         the source string as passed in by the user on the CLI or SDK.
+    no_copy_data_source
+        do not copy bucket data, create a shallow clone.
     version
         The version of the datastore to create with some same `name`.
     """
     if Path(source).exists():
-        source = ""
         source_type = V1DatastoreSourceType.EXPANDED_FILES
     elif validators.url(source) is True:
         source_type = V1DatastoreSourceType.HTTP_URL
     elif source.startswith("s3://"):
-        source_type = V1DatastoreSourceType.OBJECT_STORE
+        if no_copy_data_source is True:
+            source_type = V1DatastoreSourceType.OBJECT_STORE_REFERENCE_ONLY
+        else:
+            source_type = V1DatastoreSourceType.OBJECT_STORE
     else:
         raise ValueError(
             f"datastore source: {source} is not a known format. Expected `http` "
             f"url, `s3://` bucket path, or path to directory on local machine."
         )
 
+    if no_copy_data_source is True and not source.startswith("s3://"):
+        raise ValueError(
+            f"The no-copy option was passed when creating a datastore from {source}. "
+            "This option is only valid for s3 buckets. Please consider removing the "
+            "flag or ensuring that your path starts with `s3://`"
+        )
+
+    if source_type == V1DatastoreSourceType.EXPANDED_FILES:
+        source = ""  # expanded file source should not be set to a value in backend.
+
     resp = c.datastore_service_create_datastore(
         spec_cluster_id=cluster_id,
         body=V1CreateDatastoreRequest(
