@@ -506,7 +506,7 @@ class Run:
         Returns
         -------
         str
-            One of `['grid_search', 'random_search']`
+            One of `['grid_search', 'random_search', 'none']`
         """
         return self._strategy
 
