@@ -1,9 +1,9 @@
-import json
-from dataclasses import dataclass
 import datetime
-from pathlib import Path
-from typing import Dict, List, Optional
+import json
 import os
+from dataclasses import dataclass
+from pathlib import Path
+from typing import Dict, Optional
 
 from grid.sdk import env
 from grid.sdk._gql.queries import get_user_basic_info, get_user_teams
@@ -19,6 +19,7 @@ class User:
     is_verified: Optional[bool] = None
     is_blocked: Optional[bool] = None
     completed_signup: Optional[bool] = None
+    team_name: Optional[str] = None
 
 
 def set_default_cluster(cluster_id):
