@@ -6,6 +6,7 @@ import signal
 import time
 import warnings
 from collections import defaultdict
+from contextlib import contextmanager
 from dataclasses import dataclass, field, asdict
 from hashlib import blake2b
 from pathlib import Path
@@ -51,14 +52,33 @@ warnings.simplefilter("ignore")
 # -------------- interrupt handling ---------------
 
 done_event = Event()
+received_signal = None
 
 
 def handle_termination_signal(signum, frame):
+    global received_signal
     done_event.set()
+    received_signal = signum
 
 
-signal.signal(signal.SIGINT, handle_termination_signal)
-signal.signal(signal.SIGTERM, handle_termination_signal)
+def wrap_signals(fn):
+    global received_signal
+    prev_sigint_handler = signal.signal(signal.SIGINT, handle_termination_signal)
+    prev_sigterm_handler = signal.signal(signal.SIGTERM, handle_termination_signal)
+
+    def wrapper(*args, **kwargs):
+        try:
+            fn(*args, **kwargs)
+        except RuntimeError as e:
+            print(e)
+
+        signal.signal(signal.SIGTERM, prev_sigterm_handler)
+        signal.signal(signal.SIGINT, prev_sigint_handler)
+        if received_signal is not None:
+            signal.raise_signal(received_signal)
+
+    return wrapper
+
 
 # ----------------- data structures ----------------
 
@@ -227,15 +247,16 @@ def load_datastore_work_state(grid_dir: Path, datastore_id: str) -> Work:
     work = Work.from_dict(ujson.loads(state_file.read_text()))
     if work.grid_cli_version != __version__:
         raise RuntimeError(
-            "Cannot resume an upload which was started with a different version "
-            "of the grid. Please restart the upload to use this datastore. "
+            "An incomplete datastore upload was dected which was created from a different version "
+            "of the grid CLI. The incomplete work has been invalidated and the datastore must be "
+            "re-uploaded in full as part of a new `grid datastore create` command."
         )
     return work
 
 
 def _save_work_state(grid_dir: Path, work: Work) -> None:
     datastores_dir = grid_dir.joinpath("datastores")
-    os.makedirs(str(datastores_dir), exist_ok=True)
+    datastores_dir.mkdir(parents=True, exist_ok=True)
     state_file = datastores_dir.joinpath(f"{work.datastore_id}.json")
     state_file.touch()
     state_file.write_text(ujson.dumps(asdict(work)))
@@ -251,7 +272,7 @@ def remove_datastore_work_state(grid_dir: Path, datastore_id: str) -> None:
 def find_incomplete_datastore_upload(grid_dir: Path) -> Union[str, None]:
     """Finds an incomplete upload in the grid dir and returns the datastore id if it exsists, else None"""
     datastores_dir = grid_dir.joinpath("datastores")
-    os.makedirs(str(datastores_dir), exist_ok=True)
+    datastores_dir.mkdir(parents=True, exist_ok=True)
     for work_state_file in list(datastores_dir.iterdir()):
         if not work_state_file.is_file() or not work_state_file.name.endswith(".json"):
             continue
@@ -283,7 +304,12 @@ def _calculate_file_local_id(f: Path) -> str:
 
 
 def initialize_upload_work(
-    name: str, datastore_id: str, cluster_id: str, creation_timestamp: int, source_path: str, version: str
+    name: str,
+    datastore_id: str,
+    cluster_id: str,
+    creation_timestamp: int,
+    source_path: str,
+    version: str,
 ) -> Work:
     rel_path = Path(source_path)
     abs_path = rel_path.absolute()
@@ -379,7 +405,7 @@ def _do_upload(session: requests.Session, file_path: Path, task: UploadTask, pro
             data = file.read(task.read_range_bytes)
             bytes_read = len(data)
 
-        response = session.put(task.url, data=data, verify=False)
+        response = session.put(task.url, data=data)
         if 'ETag' not in response.headers:
             raise ValueError(f"Unexpected response from S3, response: {response.content}")
         task.etag = str(response.headers['ETag']).strip('"')
@@ -428,14 +454,39 @@ def _upload_next_batch(c: 'GridRestClient', work: Work, progress_bar: PBar) -> D
         batch_files[file.local_id] = file
 
     # We can use a with statement to ensure threads are cleaned up promptly
-    retries = Retry(total=5, status_forcelist=[500, 503])
-    with concurrent.futures.ThreadPoolExecutor(max_workers=env.UPLOAD_MAX_WORKER_THREADS
-                                               ) as executor, requests.Session() as s:
+    #
+    # The algorithm dictating backoff factor is as follows:
+    #
+    # {backoff factor} * (2 ** ({number of total retries} - 1))
+    #
+    # For example, if the backoff factor is set to:
+    #
+    # 1 second the successive sleeps will be 0.5, 1, 2, 4, 8, 16, 32, 64, 128, 256.
+    # 2 seconds - 1, 2, 4, 8, 16, 32, 64, 128, 256, 512
+    # 10 seconds - 5, 10, 20, 40, 80, 160, 320, 640, 1280, 2560
+    #
+    # To find the total time spent before erroring, sum up backoff factors for
+    # each sleep iterations up to the total number of retries (max_retries)
+    retries = Retry(
+        total=8,
+        # status_forcelist retry codes
+        # ----------------------------
+        # 104: connection reset by peer can occur when 307 redirect requests aren't received quickly enough.
+        # 429: too many requests (just need to slow down... exponential backoff handles this)
+        # 500, 502, 503, 504: assorted errors AWS uses to say "yeah... we screwed up"
+        status_forcelist=[104, 429, 500, 502, 503, 504],
+        method_whitelist=["HEAD", "GET", "PUT", "POST"],
+        backoff_factor=2,
+    )
+    max_work_threads = env.UPLOAD_MAX_WORKER_THREADS
+    with concurrent.futures.ThreadPoolExecutor(max_workers=max_work_threads) as executor, requests.Session() as s:
+        # For explanation of HTTPAdapter and request session see: https://stackoverflow.com/a/34893364
         s.mount(
             'https://',
             HTTPAdapter(
-                pool_connections=env.UPLOAD_MAX_WORKER_THREADS,
-                pool_maxsize=env.UPLOAD_MAX_WORKER_THREADS,
+                # should only ever "need" 1 pool_connection(s); but using 3 to be safe in even of redirects
+                pool_connections=3,
+                pool_maxsize=max_work_threads * 2,
                 max_retries=retries,
             )
         )
@@ -447,6 +498,8 @@ def _upload_next_batch(c: 'GridRestClient', work: Work, progress_bar: PBar) -> D
                 futures.append(executor.submit(_do_upload, s, file_path, task, progress_bar))
 
         for future in concurrent.futures.as_completed(futures):
+            if done_event.is_set():
+                raise RuntimeError("Termination signal (ie. SIGINT, SIGTERM, or SIGKILL) received")
             res = future.result()
             uploaded_file_tasks[res.data_file_local_id].append(res)
 
@@ -539,6 +592,7 @@ def _start_finalize(c: 'GridRestClient', grid_dir: Path, work: Work, progress_ba
 # ------------------------- module user methods ---------------------------
 
 
+@wrap_signals
 def resume_datastore_upload(client: 'GridRestClient', grid_dir: Path, work: Work):
     pbar = PBar()
     with Live(pbar.progress_group):
@@ -553,9 +607,16 @@ def resume_datastore_upload(client: 'GridRestClient', grid_dir: Path, work: Work
     return True
 
 
+@wrap_signals
 def begin_new_datastore_upload(
-    client: 'GridRestClient', grid_dir: Path, source_path: Path, cluster_id: str, datastore_id: str,
-    datastore_name: str, datastore_version: str, creation_timestamp: datetime.datetime
+    client: 'GridRestClient',
+    grid_dir: Path,
+    source_path: Path,
+    cluster_id: str,
+    datastore_id: str,
+    datastore_name: str,
+    datastore_version: str,
+    creation_timestamp: datetime.datetime,
 ):
     creation_timestamp = int(time.mktime(creation_timestamp.timetuple()))  # convert to unix time
 
@@ -572,6 +633,7 @@ def begin_new_datastore_upload(
         )
         pbar.setup_progress_meter(work)
 
+        _save_work_state(grid_dir, work)
         _start_upload(c=client, grid_dir=grid_dir, work=work, progress_bar=pbar)
         _start_finalize(c=client, grid_dir=grid_dir, work=work, progress_bar=pbar)
         mark_datastore_upload_complete(c=client, cluster_id=cluster_id, datastore_id=datastore_id)
